require([
  "dijit/form/Form",
  "dijit/form/Button",
  "dijit/form/RadioButton",
  "dijit/form/Select",
  "dijit/ColorPalette",
  "dijit/Dialog",
  "dijit/registry",
  "dojo/_base/array",
  "dojo/_base/declare",
  "dojo/_base/json",
  "dojo/_base/lang",
  "dojo/dom",
  "dojo/dom-construct",
  "dojo/dom-style",
  "dojo/on",
  "dojo/query",
  "dojo/store/JsonRest",
  "dojox/html/entities",
  "dojox/charting/themes/PlotKit/blue",
  "dojox/charting/themes/PlotKit/orange",
  "dojox/charting/themes/PlotKit/cyan",
  "dojox/charting/themes/PlotKit/red",
  "dojox/charting/themes/PlotKit/purple",
  "dojox/charting/themes/PlotKit/green",
  "dojox/charting/widget/Legend",
  "dojox/charting/Chart2D",
  "dojox/charting/action2d/Highlight",
  "dojox/charting/action2d/Magnify",
  "dojox/charting/action2d/MoveSlice",
  "dojox/charting/action2d/Tooltip"
], function(
  Form,
  Button,
  RadioButton,
  Select,
  ColorPalette,
  Dialog,
  registry,
  array,
  declare,
  json,
  lang,
  dom,
  domConstruct,
  domStyle,
  on,
  query,
  JsonRest,
  entities,
  blue,
  orange,
  cyan,
  red,
  purple,
  green,
  Legend,
  Chart2D,
  Highlight,
  Magnify,
  MoveSlice,
  Tooltip
) {

  declare("GraphDetailDialog", [Dialog], {
  constructor: function(initValue) {
    this.defaultSize = {};
    this.defaultSize.height = 290;
    this.defaultSize.width = 500;
    this.eventHandlers = [];
  },

  setContent: function(jshgraph) {
    var actionBar = domConstruct.create("div", {
        "id": "graphDetailDialogContents"
    }, this.containerNode);

    var contentBar = domConstruct.create("div", {}, this.containerNode);
    this.colorPalette = new ColorPalette({
      palette: "7x10",
      onChange: function(val) {
          this.dialog.plotTypeColor = val;
      }
    }).placeAt(contentBar);
    this.colorPalette.dialog = this;
    domConstruct.create("br", {}, contentBar);

    this.graphTypeSelect = new Select({
      name: chartNamePrefix + "plotTypeGraph",
      options: graphTypeOptions,
      id: chartNamePrefix + "plotTypeGraph",
      onChange: function(val) {
        this.dialog.plotTypeGraph = val;
      }
    }).placeAt(contentBar);
    this.graphTypeSelect.dialog = this;
    domConstruct.create("br", {}, contentBar);

    var chartNamePrefix = jshgraph.chartNamePrefix;
    this.axisSelectSeries1 = new RadioButton({
        name: chartNamePrefix + "plotTypeAxis",
        value: "series1",
        id: chartNamePrefix + "plotTypeAxis_series1",
        onClick: function(evt) {
          this.dialog.plotTypeAxis = "series1";
        }
    }).placeAt(contentBar);
    this.axisSelectSeries1.dialog = this;
    var labelSeries1 = domConstruct.create("label", {
      "for": chartNamePrefix + "plotTypeAxis_series1"
    }, contentBar);
    labelSeries1.innerHTML = jshgraph.getMessage("plotDialogDisplayMeasureSeries1Label");

    this.axisSelectSeries2 = new RadioButton({
      name: chartNamePrefix + "plotTypeAxis",
      value: "series2",
      id: chartNamePrefix + "plotTypeAxis_series2",
      onClick: function(evt) {
        this.dialog.plotTypeAxis = "series2";
      }
    }).placeAt(contentBar);
    this.axisSelectSeries2.dialog = this;
    var labelSeries2 = domConstruct.create("label", {
      "for": chartNamePrefix + "plotTypeAxis_series2"
    }, contentBar);
    labelSeries2.innerHTML = jshgraph.getMessage("plotDialogDisplayMeasureSeries2Label");

    this.axisSelectNone = new RadioButton({
      name: chartNamePrefix + "plotTypeAxis",
      value: "none",
      id: chartNamePrefix + "plotTypeAxis_none",
      onClick: function(evt) {
        this.dialog.plotTypeAxis = "none";
      }
    }).placeAt(contentBar);
    this.axisSelectNone.dialog = this;
    var labelNone = domConstruct.create("label", {
      "for": chartNamePrefix + "plotTypeAxis_none"
    }, contentBar);
    labelNone.innerHTML = jshgraph.getMessage("plotDialogDisplayMeasureNoneLabel");

    this.buttonOk = new Button({
      label: jshgraph.getMessage("plotDialogOkLabel"),
      onClick: function(evt) {
        this.dialog.hide();
        var isChanged = false;
        if (this.dialog.plotTypeColor != null) {
          this.dialog.jshgraph.plotTypeColor[this.dialog.plotTypeName] = this.dialog.plotTypeColor;
          isChanged = true;
        }
        if (this.dialog.plotTypeGraph != null) {
          if (this.dialog.jshgraph.type == this.dialog.plotTypeGraph) {
            delete this.dialog.jshgraph.plotTypeGraph[this.dialog.plotTypeName];
          } else {
            this.dialog.jshgraph.plotTypeGraph[this.dialog.plotTypeName] = this.dialog.plotTypeGraph;
          }
          isChanged = true;
        }
        if (this.dialog.plotTypeAxis != null) {
          this.dialog.jshgraph.plotTypeAxis[this.dialog.plotTypeName] = this.dialog.plotTypeAxis;
          isChanged = true;
        }
        if (isChanged) {
          this.dialog.jshgraph.changeType(this.dialog.jshgraph.type);
        }
      }
    }).placeAt(this.containerNode);
    this.buttonOk.dialog = this;

    this.buttonCancel = new Button({
      label: jshgraph.getMessage("plotDialogCancelLabel"),
      onClick: function(evt) {
        this.dialog.hide();
      }
    }).placeAt(this.containerNode);
    this.buttonCancel.dialog = this;

    this.buttonReset = new Button({
      label: jshgraph.getMessage("plotDialogResetLabel"),
      onClick: function(evt) {
        delete this.dialog.jshgraph.plotTypeColor[this.dialog.plotTypeName];
        delete this.dialog.jshgraph.plotTypeGraph[this.dialog.plotTypeName];
        delete this.dialog.jshgraph.plotTypeAxis[this.dialog.plotTypeName];
        this.dialog.jshgraph.changeType(this.dialog.jshgraph.type);
        this.dialog.hide();
      }
    }).placeAt(this.containerNode);
    this.buttonReset.dialog = this;
  },

  setParam: function(param) {
    var s = param.name;
    var gddc = dom.byId("graphDetailDialogContents");
    gddc.innerHTML = s;
    this.plotTypeName = param.name;
    this.jshgraph = param.jshgraph;

    var options = this.graphTypeSelect.options;
    if (this.jshgraph.arrayIndexOf(this.jshgraph.verticalGraphTypes, this.jshgraph.type) >= 0) {
      for (var i=0; i<options.length; i++) {
        if (this.jshgraph.arrayIndexOf(this.jshgraph.verticalGraphTypes, options[i].value) >= 0) {
            options[i].disabled = false;
        } else {
            options[i].disabled = true;
        }
      }
    } else if (this.jshgraph.arrayIndexOf(this.jshgraph.horizontalGraphTypes, this.jshgraph.type) >= 0) {
      for (var i=0; i<options.length; i++) {
        if (this.jshgraph.arrayIndexOf(this.jshgraph.horizontalGraphTypes, options[i].value) >= 0) {
            options[i].disabled = false;
        } else {
            options[i].disabled = true;
        }
      }
    }
    this.graphTypeSelect.startup();

    this.plotTypeColor = null;

    var plotTypeGraph = this.jshgraph.plotTypeGraph[this.plotTypeName];
    if (plotTypeGraph != null) {
        this.graphTypeSelect.setValue(plotTypeGraph);
    } else {
        this.graphTypeSelect.setValue(this.jshgraph.type);
    }
    this.plotTypeGraph = null;

    var plotTypeAxis = this.jshgraph.plotTypeAxis[this.plotTypeName];
    if (plotTypeAxis == "series1") {
        this.axisSelectSeries1.setValue(true);
    } else {
        this.axisSelectSeries1.setValue(false);
    }
    if (plotTypeAxis == "series2") {
        this.axisSelectSeries2.setValue(true);
    } else {
        this.axisSelectSeries2.setValue(false);
    }
    if (plotTypeAxis == "none") {
        this.axisSelectNone.setValue(true);
    } else {
        this.axisSelectNone.setValue(false);
    }
    this.plotTypeAxis = null;
  },

  });

  declare("JshGraph", null, {
    constructor: function(initialValue, TypeDisplayedValue, messageResource) {
      this.isStopChangeEvent = true;
      this.setInitialValue(initialValue, TypeDisplayedValue, messageResource);

      // 複数グラフが存在する場合、float 属性を設定して横並びにする
      var chartWrappers = query(".chart_wrapper");
      if (chartWrappers.length > 1) {
        array.forEach(chartWrappers, function(node, index, arr){
          domStyle.set(node, "float", "left");
        });
      }
      if (this.graphDetailDialog == null) {
        this.graphDetailDialog = new GraphDetailDialog({
          name: "graphDetail",
          title: this.getMessage("plotDialogTitleLabel")
        });
        this.graphDetailDialog.setContent(this);
      }
    },

    setInitialValue: function(initialValue, TypeDisplayedValue, messageResource) {
      this.chartNamePrefix = initialValue.chartNamePrefix;
      this.default_type = initialValue.type;
      this.default_w = initialValue.width;
      this.default_h = initialValue.height;
      this.default_theme = eval(initialValue.theme);
      this.default_themename = initialValue.themename;
      this.default_PieFontSize = initialValue.pieFontSize;
      this.default_PieLabelLength = initialValue.pieLabelLength;
      this.default_GraphFontSize = initialValue.graphFontSize;
      this.default_ReverseDimension = initialValue.reverseDimension;
      this.default_SortDimension = initialValue.sortDimension;
      this.orig_datas_item = [].concat(initialValue.datas.item);
      this.datas = initialValue.datas;
      this.graphSettingFormId = initialValue.chartNamePrefix + "graphSettingForm";
      this.graphTypeSelectorId = initialValue.chartNamePrefix + "type";
      this.chartContainerId = initialValue.chartNamePrefix + "chart_container";
      this.chartId = initialValue.chartNamePrefix + "chart";
      this.themeSelectorId = initialValue.chartNamePrefix + "theme";
      this.chartWidthTextboxId = initialValue.chartNamePrefix + "chart_width";
      this.chartHeightTextboxId = initialValue.chartNamePrefix + "chart_height";
      this.pieFontSizeTextboxId = initialValue.chartNamePrefix + "pie_font_size";
      this.pieLabelLengthTextboxId = initialValue.chartNamePrefix + "pie_label_length";
      this.graphFontSizeTextboxId = initialValue.chartNamePrefix + "graph_font_size";
      this.showChartButtonId = initialValue.chartNamePrefix + "showChartButton";
      this.detailSettingButtonId = initialValue.chartNamePrefix + "detailSettingButton";
      this.reverseDimensionId = initialValue.chartNamePrefix + "reverseDimension";
      this.sortDimensionId = initialValue.chartNamePrefix + "sortDimension";
      this.graphDetailId = initialValue.chartNamePrefix + "graphDetail";
      this.graphDetailButtonId = initialValue.chartNamePrefix + "graphDetailButton";
      this.deleteSettingButtonId = initialValue.chartNamePrefix + "deleteSettingButton";
      this.sendAllSettingButtonId = initialValue.chartNamePrefix + "sendAllSettingButton";
      this.TypeDisplayedValue = TypeDisplayedValue;
      this.messageResource = messageResource;
      this.dimensionItemName = initialValue.dimensionItemName;
      this.measureItemNames = initialValue.measureItemNames;
      this.screenid = initialValue.screenid;
      this.userid = initialValue.userid;
      this.allsetting = false;
      this.legend = null;
      this.verticalGraphTypes = [
        "column",
        "clusteredColumn",
        "stackedColumn",
        "line",
        "mline",
        "stackedLine",
        "scatter",
        "area",
        "stackedArea"
      ];
      this.horizontalGraphTypes = [ "bar", "clusteredBar", "stackedBar" ];
      this.pieGraphTypes = [ "pie", "piev" ]
      this.magnifyGraphTypes = [ "mline", "scatter" ];
      this.graphTypeToPlot = {
        "column": "Columns",
        "clusteredColumn": "ClusteredColumns",
        "stackedColumn": "StackedColumns",
        "line": "Lines",
        "mline": "Markers",
        "stackedLine": "StackedLines",
        "scatter": "Scatter",
        "area": "Areas",
        "stackedArea": "StackedAreas",
        "bar": "Bars",
        "clusteredBar": "ClusteredBars",
        "stackedBar": "StackedBars"
      };
      this.setDefaultValue();
    },

    arrayIndexOf: function(arr, elem) {
      for(var i = 0; i < arr.length; i++){
        if(arr[i] === elem){
          return i;
        };
      }
      return -1;
    },

    setDefaultValue: function() {
      this.type = this.default_type;
      this.w = this.default_w;
      this.h = this.default_h;
      this.theme = this.default_theme;
      this.themename = this.default_themename;
      this.PieFontSize = this.default_PieFontSize;
      this.PieLabelLength = this.default_PieLabelLength;
      this.GraphFontSize = this.default_GraphFontSize;
      this.ReverseDimension = this.default_ReverseDimension;
      this.SortDimension = this.default_SortDimension;
      var __isStopEvent = this.isStopChangeEvent;
      this.isStopChangeEvent = true;
      var tmp_showChartButton = this.getShowChartButtonChecked();
      var tmp_detailSettingButton = this.getDetailSettingButton();
      dom.byId(this.graphSettingFormId).reset();
      this.setShowChartButtonChecked(tmp_showChartButton);
      this.setDetailSettingButton(tmp_detailSettingButton);
      this.isStopChangeEvent = __isStopEvent;
      this.plotTypeAxis = {};
      this.plotTypeGraph = {};
      this.plotTypeColor = {};
      this.setting = this.createSettingJson();
    },

    // addEvent
    setEventFunctionToSettingControls: function() {
      on(registry.byId(this.graphTypeSelectorId), "change", lang.hitch(this, this.changeType));
      on(registry.byId(this.showChartButtonId), "click", lang.hitch(this, this.showChart));
      on(registry.byId(this.detailSettingButtonId), "click", lang.hitch(this, this.showDetailSettings));
      on(registry.byId(this.themeSelectorId), "change", lang.hitch(this, this.changeTheme));
      on(registry.byId(this.chartWidthTextboxId), "change", lang.hitch(this, this.changeSize));
      on(registry.byId(this.chartHeightTextboxId), "change", lang.hitch(this, this.changeSize));
      on(registry.byId(this.pieFontSizeTextboxId), "change", lang.hitch(this, this.changePieFontSize));
      on(registry.byId(this.pieLabelLengthTextboxId), "change", lang.hitch(this, this.changePieLabelLength));
      on(registry.byId(this.graphFontSizeTextboxId), "change", lang.hitch(this, this.changeGraphFontSize));
      on(registry.byId(this.reverseDimensionId), "click", lang.hitch(this, this.changeReverseDimension));
      on(registry.byId(this.sortDimensionId), "click", lang.hitch(this, this.changeSortDimension));
      on(registry.byId(this.graphDetailButtonId), "click", lang.hitch(this, this.openGraphDetailDialog));
      on(registry.byId(this.deleteSettingButtonId), "click", lang.hitch(this, this.deleteSettingJson));
      var sendAllSettingButton = registry.byId(this.sendAllSettingButtonId);
      if (sendAllSettingButton != null) {
        on(sendAllSettingButton, "click", lang.hitch(this, this.sendAllSettingJson));
      }
      this.isStopChangeEvent = false;
      this.makeObjects();
    },

    showChart: function() {
      if (this.getShowChartButtonChecked()) {
        dom.byId(this.chartNamePrefix + "chart_view").style.display = "inline";
        registry.byId(this.showChartButtonId).setLabel(this.getMessage("showChartButtonCheckedLabel"));
        this.changeType(this.type);
      } else {
        dom.byId(this.chartNamePrefix + "chart_view").style.display = "none";
        registry.byId(this.showChartButtonId).setLabel(this.getMessage("showChartButtonUnCheckedLabel"));
        this.clearChart();
      }
    },

    showDetailSettings: function() {
      if (this.getDetailSettingButton()) {
        dom.byId(this.chartNamePrefix + "detail_setting").style.display = "inline";
        registry.byId(this.detailSettingButtonId).setLabel(this.getMessage("detailSettingButtonCheckedLabel"));
      } else {
        dom.byId(this.chartNamePrefix + "detail_setting").style.display = "none";
        registry.byId(this.detailSettingButtonId).setLabel(this.getMessage("detailSettingButtonUnCheckedLabel"));
      }
    },

    getMeasureCount: function() {
      var measureCount = 0;
      for (var i=0;i<this.measureItemNames.length;i++) {
        if (this.datas.item[0][this.measureItemNames[i]] instanceof Array) {
          measureCount += this.datas.item[0][this.measureItemNames[i]].length;
        } else {
          measureCount++;
        }
      }
      return measureCount;
    },

    getLabels: function() {
      if (! this.ReverseDimension) {
        return this.getDimLabels();
      } else {
        return this.getMeaLabels();
      }
    },

    getMeaLabels: function() {
      var labels = [];
      for (var i=0; i<this.datas.item.length; i++) {
        var value = this.getDimContent(this.datas.item[i]);
        var num = parseInt(value);
        value = dojox.html.entities.encode(value);
        if (! isNaN(num)) {
          labels.push({value:i+1, text:value});
        } else {
          labels.push({value:i+1, text:value});
        }
      }
      return labels;
    },

    getDimLabels: function() {
      var labels = [];
      var index = 1;
      for (var i=0;i<this.measureItemNames.length;i++) {
        var measureItemName = this.measureItemNames[i];
        var item = this.datas.item[0][measureItemName];
        if (item instanceof Array) {
          for (var j=0;j<item.length;j++) {
            var value = dojox.html.entities.encode(item[j].label);
            labels.push({value:index, text:value});
            index++;
          }
        } else {
          var value = dojox.html.entities.encode(item.label);
          labels.push({value:index, text:value});
          index++;
        }
      }
      return labels;
    },

    getSdata: function(item) {
      var sdata = [];
      for (var i=0;i<this.measureItemNames.length;i++) {
        var measureItemName = this.measureItemNames[i];
        this.pushMeaItemContent(sdata, measureItemName, item);
      }
      return sdata;
    },

    getSdataMea: function(measureItemName, sname_dim, items) {
      var sdata = [];
      for (i=0; i<sname_dim.length; i++) {
        for (j=0; j<items.length; j++) {
          var value = this.getDimContent(items[j]);
          value = dojox.html.entities.encode(value);
          if (value != sname_dim[i].text) {
              continue;
          }
          this.pushMeaItemContent(sdata, measureItemName, items[j]);
        }
      }
      return sdata;
    },

    setPlotSettings: function(xaxis, yaxis, secondaxis, defaultplot, backGrid) {
      this.add_graph_defaultplot(xaxis);
      this.chart.addAxis("x", xaxis);
      this.add_graph_defaultplot(yaxis);
      this.chart.addAxis("y", yaxis);
      this.add_graph_defaultplot(secondaxis);
      this.chart.addAxis("second", secondaxis);
      this.chart.addPlot("back_grid", backGrid);
      this.add_graph_defaultplot(defaultplot);
      var plotname = "default";
      this.chart.addPlot(plotname, defaultplot);
      new dojox.charting.action2d.Tooltip(this.chart, plotname);
      new dojox.charting.action2d.Highlight(this.chart, plotname);
      if (this.arrayIndexOf(this.magnifyGraphTypes, this.plot) >= 0) {
        new dojox.charting.action2d.Magnify(this.chart, plotname);
      }
      this.chart.connectToPlot(plotname, this.callPlotEvent);
      // use callPlotEvent
      this.chart.jshgraph = this;
    },

    setSeries2PlotSettings: function(plottypename) {
      var series2plot = {gap: 5}
      var plottype;
      if (this.plotTypeGraph[plottypename] != null) {
        plottype = this.plotTypeGraph[plottypename];
      } else {
        plottype = this.type;
      }
      series2plot["type"] = this.graphTypeToPlot[plottype];
      if (this.plotTypeAxis[plottypename] == "series2") {
        if (this.arrayIndexOf(this.verticalGraphTypes, this.type) >= 0) {
          series2plot["vAxis"] = "second";
        } else if (this.arrayIndexOf(this.horizontalGraphTypes, this.type) >= 0) {
          series2plot["hAxis"] = "second";
        }
      }
      this.add_graph_defaultplot(series2plot);
      var plotname = "plot_"+plottypename
      this.chart.addPlot(plotname, series2plot);
      new dojox.charting.action2d.Tooltip(this.chart, plotname);
      new dojox.charting.action2d.Highlight(this.chart, plotname);
      if (this.arrayIndexOf(this.magnifyGraphTypes, series2plot["type"]) >= 0) {
        new dojox.charting.action2d.Magnify(this.chart, plotname);
      }
      this.chart.connectToPlot(plotname, this.callPlotEvent);
      this.chart.movePlotToFront(plotname);
    },

    getDimContent: function(item) {
      return this.__getDimContent(this.dimensionItemName, item);
    },

    pushMeaItemContent: function(sdata, measureItemName, item) {
      if (item[measureItemName] instanceof Array) {
        for (j=0;j<item[measureItemName].length;j++) {
          this.pushMeaItemContentValue(sdata, item[measureItemName][j].content);
        }
      } else {
        this.pushMeaItemContentValue(sdata, item[measureItemName].content);
      }
    },

    pushMeaItemContentValue: function(sdata, value) {
      var valuenum = Number(value)
      if (Number.isNaN(valuenum)) {
        sdata.push(value);
      } else {
        sdata.push(valuenum);
      }
    }, 

    setPlotData: function() {
      var sname_dim = this.getMeaLabels();
      this.series = []
      if (! this.ReverseDimension) {
        for (i=0; i<this.datas.item.length; i++) {
          var sdatas_dim = this.getSdata(this.datas.item[i]);
          var sname_dim_text = sname_dim[i].text;
          this.series.push({label:sname_dim_text, value:sname_dim_text});
          if (this.plotTypeAxis[sname_dim_text] == "none") {
              continue;
          }
          var seriesOptions = {};
          var hasSeriesOptions = false;
          if (this.plotTypeAxis[sname_dim_text] == "series2"
                  || this.plotTypeGraph[sname_dim_text] != null) {
              this.setSeries2PlotSettings(sname_dim_text);
              seriesOptions["plot"] = "plot_"+sname_dim_text;
              hasSeriesOptions = true;
          }
          if (this.plotTypeColor[sname_dim_text] != null) {
              seriesOptions["fill"] = this.plotTypeColor[sname_dim_text];
              seriesOptions["stroke"] = {
                  color: this.plotTypeColor[sname_dim_text]
              };
              hasSeriesOptions = true;
          }
          if (hasSeriesOptions) {
              this.chart.addSeries(sname_dim_text, sdatas_dim, seriesOptions);
          } else {
              this.chart.addSeries(sname_dim_text, sdatas_dim);
          }
        }
      } else {
        var sname_mea = this.getDimLabels();
        for (var j=0; j<this.measureItemNames.length; j++) {
          var measureItemName = this.measureItemNames[j];
          var sdatas_mea = this.getSdataMea(measureItemName, sname_dim, this.datas.item);
          var sname_mea_text = sname_mea[j].text;
          this.series.push({label:sname_mea_text, value:sname_mea_text});
          if (this.plotTypeAxis[sname_mea_text] == "none") {
              continue;
          }
          var seriesOptions = {};
          var hasSeriesOptions = false;
          if (this.plotTypeAxis[sname_mea_text] == "series2"
                  || this.plotTypeGraph[sname_mea_text] != null) {
              this.setSeries2PlotSettings(sname_mea_text);
              seriesOptions["plot"] = "plot_"+sname_mea_text;
              hasSeriesOptions = true;
          }
          if (this.plotTypeColor[sname_mea_text] != null) {
              seriesOptions["fill"] = this.plotTypeColor[sname_mea_text];
              seriesOptions["stroke"] = {
                  color: this.plotTypeColor[sname_mea_text]
              };
              hasSeriesOptions = true;
          }
          if (hasSeriesOptions) {
            this.chart.addSeries(sname_mea_text, sdatas_mea, seriesOptions);
          } else {
            this.chart.addSeries(sname_mea_text, sdatas_mea);
          }
        }
      }
    },

    sortData: function() {
      if (! this.SortDimension) {
        var that = this;
        this.datas.item.sort(lang.hitch(that, this.comparatorDatasItem));
      } else {
        this.datas.item = [].concat(this.orig_datas_item);
      }
    },

    make_column: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "Columns", gap: 5};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_clusteredColumn: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "ClusteredColumns", gap: 5};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_stackedColumn: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "StackedColumns", gap: 5};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_bar: function() {
      var xaxis = {fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var yaxis = {vertical:true, fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var secondaxis = {leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "Bars", gap: 5};
      var backGrid = {type: "Grid", vMajorLines: true, hMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_clusteredBar: function() {
      var xaxis = {fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var yaxis = {vertical:true, fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var secondaxis = {leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "ClusteredBars", gap: 5};
      var backGrid = {type: "Grid", vMajorLines: true, hMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_stackedBar: function() {
      var xaxis = {fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var yaxis = {vertical:true, fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var secondaxis = {leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "StackedBars", gap: 5};
      var backGrid = {type: "Grid", vMajorLines: true, hMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    getPieLabel: function(items) {
      var measures = items[0][this.measureItemNames[0]];
      if (measures instanceof Array) {
        return measures[0].label;
      } else {
        return measures.label;
      }
    },

    getPieData: function(items) {
      var measures = items[0][this.measureItemNames[0]];
      var sums = [];
      if (measures instanceof Array) {
        for (var i=0;i<items.length;i++) {
          for (var j=0;j<items[i][this.measureItemNames[0]].length;j++) {
            sums.push(items[i][this.measureItemNames[0]][j].content);
          }
        }
      } else {
        for (var i=0;i<items.length;i++) {
          sums.push(items[i][this.measureItemNames[0]].content);
        }
      }
      return sums;
    },

    make_pie: function() {
      var sname = this.getPieLabel(this.datas.item);
      this.series = [ {label:sname, value:sname} ];
      var sumarray = this.getPieData(this.datas.item);
      var sdatas = new Array(this.datas.item.length);
      for (i=0; i<sdatas.length; i++) {
        var txt;
        if (this.datas.item[i][this.dimensionItemName] instanceof Array) {
          if (this.datas.item[i][this.dimensionItemName][0].id != undefined) {
            txt = this.datas.item[i][this.dimensionItemName][0].label;
          } else {
            txt = this.datas.item[i][this.dimensionItemName][0].content;
          }
        } else {
          if (this.datas.item[i][this.dimensionItemName].id != undefined) {
            txt = this.datas.item[i][this.dimensionItemName].label;
          } else {
            txt = this.datas.item[i][this.dimensionItemName].content;
          }
        }
        if (txt.length > this.PieLabelLength) {
          txt = txt.substring(0, this.PieLabelLength) + "...";
        }
        sdatas[i] = {tooltip:sumarray[i], y:Number(sumarray[i]), text:txt};
      }
      this.chart.addPlot("default", this.get_pie_defaultplot());
      new dojox.charting.action2d.Tooltip(this.chart,"default");
      new dojox.charting.action2d.MoveSlice(this.chart, "default");
      this.chart.addSeries(sname, sdatas);
    },

    make_piev: function() {
      var sname = this.getPieLabel(this.datas.item);
      this.series = [ {label:sname, value:sname} ];
      var measureItemName = this.measureItemNames[0];
      var sdatas;
      if (this.datas.item[0][measureItemName] instanceof Array) {
        var sumarray = this.getPieData(this.datas.item);
        sdatas = new Array(this.datas.item[0][measureItemName].length);
        for (i=0; i<sdatas.length; i++) {
          var txt;
          if (this.datas.item[0][measureItemName][i].id != undefined) {
            txt = this.datas.item[0][measureItemName][i].label;
          } else {
            txt = sumarray[i].toString();
          }
          if (txt.length > this.PieLabelLength) {
            txt = txt.substring(0, this.PieLabelLength)+"...";
          }
          sdatas[i] = {tooltip:sumarray[i], y:Number(sumarray[i]), text:txt};
        }
      } else {
        var sum = 0;
        for (i=0; i<this.datas.item.length; i++) {
          sum += parseInt(this.datas.item[i][measureItemName].content);
        }
        sdatas = new Array(1);
        var txt;
        if (this.datas.item[0][measureItemName].id != undefined) {
          txt = this.datas.item[0][measureItemName].label;
        } else {
          txt = sum.toString();
        }
        if (txt.length > this.PieLabelLength) {
          txt = txt.substring(0, this.PieLabelLength)+"...";
        }
        sdatas[0] = {tooltip:sum, y:sum, text:txt};
      }
      this.chart.addPlot("default", this.get_pie_defaultplot());
      new dojox.charting.action2d.Tooltip(this.chart,"default");
      new dojox.charting.action2d.MoveSlice(this.chart, "default");
      this.chart.addSeries(sname, sdatas);
    },

    make_line: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "Lines"};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_mline: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "Markers"};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_stackedLine: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "StackedLines"};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_scatter: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "Scatter"};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_area: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "Areas"};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    make_stackedArea: function() {
      var xaxis = {fixLower:"minor", fixUpper:"minor", labels:this.getLabels(), majorTickStep: 1, minorTicks: false};
      var yaxis = {vertical:true, fixLower:"major", fixUpper:"major", includeZero:true, natural:true};
      var secondaxis = {vertical: true, leftBottom: false, fixUpper: "major", includeZero: true};
      var defaultplot = {type: "StackedAreas"};
      var backGrid = {type: "Grid", hMajorLines: true, vMajorLines: false};
      this.setPlotSettings(xaxis, yaxis, secondaxis, defaultplot, backGrid);
      this.setPlotData();
    },

    clearChart: function() {
      if (this.chart != null) {
        dom.byId(this.chartContainerId).removeChild(dom.byId(this.chartId));
        window.location.href="#" + this.chartNamePrefix + "graph";
        this.chart = null;
      }
    },

    changeType: function(typename) {
      if (this.isStopChangeEvent) return;
      this.type = typename;
      var dummy = this.makeObjects();
  
      if (this.type == "pie" || this.type == "piev") {
        domStyle.set(dom.byId(this.chartId+"_legend"), "visibility", "hidden");
      } else {
        domStyle.set(dom.byId(this.chartId+"_legend"), "visibility", "visible");
      }
      this.sendSettingJson();
    },

    changeSize: function() {
      if (this.isStopChangeEvent) return;
      var w = Number(dom.byId(this.chartWidthTextboxId).value);
      var h = Number(dom.byId(this.chartHeightTextboxId).value);
      if (w >= 100 && h >= 100) {
        if (this.chart == null) return;
        this.chart.resize(w, h);
        if (w > 500) {
  	    domStyle.set(dom.byId(this.chartNamePrefix + "chart_wrapper"), "width", w);
        } else {
  	    domStyle.set(dom.byId(this.chartNamePrefix + "chart_wrapper"), "width", 500);
        }
      } else {
        window.showAlertDialog(this.getMessage("graphLabelErrorMessage"));
      }
      this.sendSettingJson();
    },

    changeTheme: function(thememodule) {
      if (this.isStopChangeEvent) return;
      this.theme = eval(thememodule);
      this.themename = dom.byId(this.themeSelectorId).value;
      if (this.chart == null) return;
      this.chart.setTheme(this.theme);
      this.chart.render();
      if (this.legend) {
        this.legend.refresh();
      }
      this.sendSettingJson();
    },

    changePieFontSize: function() {
      if (this.isStopChangeEvent) return;
      var fontsizestr = dom.byId(this.pieFontSizeTextboxId).value;
      if (fontsizestr == "") {
        this.PieFontSize = null;
        this.changeType(this.type);
        return;
      }
      var fontsize = parseInt(fontsizestr);
      if (fontsize >= 1) {
        this.PieFontSize = fontsize;
        this.changeType(this.type);
      } else {
        window.showAlertDialog(this.getMessage("graphLabelIllegalFontSizeMessage"));
        dom.byId(this.pieFontSizeTextboxId).value = this.PieFontSize;
      }
    },

    changePieLabelLength: function() {
      if (this.isStopChangeEvent) return;
      var labellength = parseInt(dom.byId(this.pieLabelLengthTextboxId).value);
      if (labellength >= 1) {
        this.PieLabelLength = labellength;
        this.changeType(this.type);
      } else {
        window.showAlertDialog(this.getMessage("graphLabelIllegalLabelLengthMessage"));
        dom.byId(this.pieLabelLengthTextboxId).value = this.PieLabelLength;
      }
    },

    changeGraphFontSize: function() {
      if (this.isStopChangeEvent) return;
      var fontsizestr = dom.byId(this.graphFontSizeTextboxId).value;
      if (fontsizestr == "") {
        this.GraphFontSize = null;
        this.changeType(this.type);
        return;
      }
      var fontsize = parseInt(fontsizestr);
      if (fontsize >= 1) {
        this.GraphFontSize = fontsize;
        this.changeType(this.type);
      } else {
        window.showAlertDialog(this.getMessage("graphLabelIllegalFontSizeMessage"));
        dom.byId(this.graphFontSizeTextboxId).value = this.GraphFontSize;
      }
    },

    changeReverseDimension: function() {
      if (this.isStopChangeEvent) return;
      this.ReverseDimension = dom.byId(this.reverseDimensionId).checked;
      this.changeType(this.type);
    },

    changeSortDimension: function() {
      if (this.isStopChangeEvent) return;
      this.SortDimension = dom.byId(this.sortDimensionId).checked;
      this.changeType(this.type);
    },

    openGraphDetailDialog: function(evt) {
      var v = dom.byId(this.graphDetailId).textContent;
      this.graphDetailDialog.setParam({
        index: -1, name: v, jshgraph: this
      });
      this.graphDetailDialog.show();
    },

    get_pie_defaultplot: function() {
      var defaultplot;
      if (this.PieFontSize != null) {
        var font = "normal normal bold " + this.PieFontSize + "pt Tahoma";
        defaultplot = {
          type: "Pie",
          font: font,
          fontColor: "black",
          labelOffset: -25,
          precision: 0
        };
      } else {
        defaultplot = {
          type: "Pie",
          labelOffset: -25,
          precision: 0
        };
      }
      return defaultplot;
    },

    add_graph_defaultplot: function(defaultplot) {
      if (this.GraphFontSize != null) {
        var font = "normal normal bold " + this.GraphFontSize + "pt Tahoma";
        defaultplot["font"] = font;
        defaultplot["fontColor"] = "black";
      }
    },

    makeObjects: function() {
      if (!this.getShowChartButtonChecked()) {
        registry.byId(this.graphTypeSelectorId).set("displayedValue", this.TypeDisplayedValue[this.type]);
        return;
      }
      var isRepaint = false;
      if (this.chart != null) {
        dom.byId(this.chartContainerId).removeChild(dom.byId(this.chartId));
        isRepaint = true;
      }
      var cnode = document.createElement("div");
      cnode.id = this.chartId;
      cnode.style.width = this.w;
      cnode.style.height = this.h;
      dom.byId(this.chartContainerId).appendChild(cnode);
      this.chart = new Chart2D(this.chartId);
      this.chart.setTheme(this.theme);
      dom.byId(this.themeSelectorId).value = this.themename;
      this.changeSize();

      this.sortData();

      switch(this.type) {
      case "clusteredColumn":
        this.make_clusteredColumn(); break;
      case "stackedColumn":
        this.make_stackedColumn(); break;
      case "bar":
        this.make_bar(); break;
      case "clusteredBar":
        this.make_clusteredBar(); break;
      case "stackedBar":
        this.make_stackedBar(); break;
      case "pie":
        this.make_pie(); break;
      case "piev":
        this.make_piev(); break;
      case "line":
        this.make_line(); break;
      case "mline":
        this.make_mline(); break;
      case "stackedLine":
        this.make_stackedLine(); break;
      case "scatter":
        this.make_scatter(); break;
      case "area":
        this.make_area(); break;
      case "stackedArea":
        this.make_stackedArea(); break;
      default:
        this.type = "column";
        this.make_column(); break;
      }
      var gd = registry.byId(this.graphDetailId);
      if (gd != null) {
        gd.destroy();
      }
      var gdp = dom.byId(this.graphDetailId+"_parent")
        new Select({
          id: this.graphDetailId,
          style: "width:200px",
          options: this.series
        }).placeAt(gdp);
      var gdb = registry.byId(this.graphDetailButtonId);
      if (this.arrayIndexOf(this.pieGraphTypes, this.type) >= 0) {
          gdb.disabled = true;
      } else {
          gdb.disabled = false;
      }
      try {
        this.chart.render();
      } catch(e) {
        console.error("chart rendering error");
        console.error(e);
      }

      if (this.type == "pie" || this.type == "piev") {
        domStyle.set(dom.byId(this.chartId+"_legend"), "visibility", "hidden");
        domStyle.set("jshgraph_config_onlypie", "display", "inline");
        domStyle.set("jshgraph_config_withoutpie", "display", "none");
      } else {
        domStyle.set(dom.byId(this.chartId+"_legend"), "visibility", "visible");
        if (this.legend != null || this.legend != undefined) {
          this.legend.destroyRecursive();
          domConstruct.place("<div id='"+this.chartId+"_legend"+"'></div>", dom.byId(this.chartId+"_view"));
        }
        this.legend = new Legend({chart: this.chart}, this.chartId+"_legend");
        domStyle.set("jshgraph_config_onlypie", "display", "none");
        domStyle.set("jshgraph_config_withoutpie", "display", "inline");
      }
      registry.byId(this.graphTypeSelectorId).set("displayedValue", this.TypeDisplayedValue[this.type]);
      if (isRepaint) {
        window.location.href="#" + this.chartNamePrefix + "_graph";
      }
    },

    callPlotEvent: function(evt) {
      // Output some debug information to the console
      //console.warn(evt.type," on element ",evt.element," with shape ",evt.shape,". index ",evt.index," evt ",evt);
      //if (evt.run) {
      //    console.warn("name ",evt.run.name);
      //}
      // Get access to the shape and type
      var shape = evt.shape, type = evt.type;
      if(type == "onclick") {
          this.graphDetailDialog.setParam({
              index: evt.index,
              name: (evt.run) ? evt.run.name : "",
              jshgraph: evt.chart.jshgraph
          });
          this.graphDetailDialog.show();
      }
    },

    getShowChartButtonChecked: function() {
      return registry.byId(this.showChartButtonId).checked;
    },

    getDetailSettingButton: function() {
      return registry.byId(this.detailSettingButtonId).checked;
    },

    setShowChartButtonChecked: function(value) {
      registry.byId(this.showChartButtonId).checked = value;
    },

    setDetailSettingButton: function(value) {
      registry.byId(this.detailSettingButtonId).checked = value;
    },

    getMessage: function(messageKey) {
      return this.messageResource[messageKey];
    },

    getUserSettingJsonQuery: function() {
      return "?jfcgraphsetting_cp$002fscreenid="+this.screenid+"&jfcgraphsetting_cp$002fuserid$005b1$005d$002f$0040id="+this.userid+"&jfcgraphsetting_cp$002fallsetting=false";
    },

    getAllSettingJsonQuery: function() {
      return "?jfcgraphsetting_cp$002fscreenid="+this.screenid+"&jfcgraphsetting_cp$002fallsetting=true";
    },

    getSettingJson: function() {
      var store = new JsonRest({
        target: "rest/jfcgraphsetting/entity"
      });
      var that = this;
      store.query(this.getUserSettingJsonQuery()).then(
        lang.hitch(that, "getSettingJsonRewrite"),
        lang.hitch(that, "getSettingJsonFailed")
      );
    },

    getSettingJsonFailed: function() {
      var that = this;
      setTimeout(lang.hitch(that, "setEventFunctionToSettingControls"), 1000);
    },

    getSettingJsonRewrite2: function(results) {
      if (results == null || results.entity == null || results.entity.length == 0) {
        var that = this;
        setTimeout(lang.hitch(that, "setEventFunctionToSettingControls"), 1000);
        return;
      }
      this.isStopChangeEvent = true;
      var result = results.entity[0];
      result.id = null;
      this.setSettingJsonInPage(result);
      var that = this;
      setTimeout(lang.hitch(that, "setEventFunctionToSettingControls"), 1000);
    },

    getSettingJsonRewrite: function(results) {
      if (results == null || results.entity == null || results.entity.length == 0) {
        var store = new JsonRest({
          target: "rest/jfcgraphsetting/entity"
        });
        var that = this;
        store.query(this.getAllSettingJsonQuery()).then(
          lang.hitch(that, "getSettingJsonRewrite2")
        );
        return;
      }
      this.isStopChangeEvent = true;
      var result = results.entity[0];
      this.setSettingJsonInPage(result);
      var that = this;
      setTimeout(lang.hitch(that, "setEventFunctionToSettingControls"), 1000);
    },

    setSettingJsonInPage: function(result) {
      this.setting = lang.clone(result);
      this.allsetting = json.fromJson(result.allsetting);
      this.plotTypeAxis = json.fromJson(result.plotTypeAxis);
      this.plotTypeGraph = json.fromJson(result.plotTypeGraph);
      this.plotTypeColor = json.fromJson(result.plotTypeColor);
      registry.byId(this.graphTypeSelectorId).setValue(result.graphType);
      this.type = result.graphType;

      var rtheme = result.theme;
      if (this.isValidTheme(rtheme)) {
        registry.byId(this.themeSelectorId).setValue(rtheme);
        this.theme = eval(rtheme);
      }
      registry.byId(this.chartWidthTextboxId).setValue(result.chartWidth);
      registry.byId(this.chartHeightTextboxId).setValue(result.chartHeight);
      registry.byId(this.pieFontSizeTextboxId).setValue(result.pieFontSize);
      this.PieFontSize = parseInt(result.pieFontSize);

      registry.byId(this.pieLabelLengthTextboxId).setValue(result.pieLabelLength);
      this.PieLabelLength = parseInt(result.pieLabelLength);

      registry.byId(this.graphFontSizeTextboxId).setValue(result.graphFontSize);
      this.GraphFontSize = parseInt(result.graphFontSize);

      var rReverseDimension = json.fromJson(result.reverseDimension);
      registry.byId(this.reverseDimensionId).setValue(rReverseDimension);
      this.ReverseDimension = rReverseDimension;

      var rSortDimension;
      if (result.sortDimension == 'true') {
          rSortDimension = true;
      } else {
          rSortDimension = false;
      }
      registry.byId(this.sortDimensionId).setValue(rSortDimension);
      this.SortDimension = rSortDimension;
    },

    isValidTheme: function(rtheme) {
      var header = "dojox.charting.themes.PlotKit.";
      if (rtheme.indexOf(header) != 0) {
          return false;
      }
      var a = rtheme.substring(header.length);
      if (! /[a-z]+/.test(a)) { // purple, green, cyan, blue, red, orange
          return false;
      }
      return true;
    },

    createSettingJson: function() {
      return {
        screenid: this.screenid,
        userid: this.userid,
        useridjshparam: this.userid,
        allsetting: json.toJson(false),
        plotTypeAxis: json.toJson(this.plotTypeAxis),
        plotTypeGraph: json.toJson(this.plotTypeGraph),
        plotTypeColor: json.toJson(this.plotTypeColor),
        graphType: registry.byId(this.graphTypeSelectorId).value,
        theme: registry.byId(this.themeSelectorId).value,
        chartWidth: dom.byId(this.chartWidthTextboxId).value,
        chartHeight: dom.byId(this.chartHeightTextboxId).value,
        pieFontSize: dom.byId(this.pieFontSizeTextboxId).value,
        pieLabelLength: dom.byId(this.pieLabelLengthTextboxId).value,
        graphFontSize: dom.byId(this.graphFontSizeTextboxId).value,
        reverseDimension: json.toJson(dom.byId(this.reverseDimensionId).checked),
        sortDimension: json.toJson(dom.byId(this.sortDimensionId).checked)
      };
    },

    sendSettingJson: function() {
      var sendSettingJson_jsono = this.createSettingJson();
      var store = new JsonRest({
          target: "rest/jfcgraphsetting/entity"
      });
      var a1 = this;
      if (! this.isDifferentSetting(sendSettingJson_jsono, this.setting)) {
        return;
      }
      if (this.setting.id == null) {
        store.query(this.getUserSettingJsonQuery()).then(function(results) {
          if (results == null || results.entity == null || results.entity.length == 0) {
            if (a1.isDifferentSetting(sendSettingJson_jsono, a1.setting)) {
              var process = store.add(sendSettingJson_jsono);
              a1.setting = sendSettingJson_jsono;
              a1.allsetting = false;
              process.then(function(results2) {
                a1.setting = results2.entity;
              });
            }
          } else {
            var result = results.entity[0];
            if (a1.isDifferentSetting(sendSettingJson_jsono, result)) {
              sendSettingJson_jsono.id = result.id;
              var process = store.put(sendSettingJson_jsono, { id: sendSettingJson_jsono.id });
              a1.setting = sendSettingJson_jsono;
              a1.allsetting = false;
              process.then(function(results2) {
                a1.setting = results2.entity;
              });
            } else {
                a1.setting = result;
            }
          }
        });
      } else {
        sendSettingJson_jsono.id = this.setting.id;
        var process = store.put(sendSettingJson_jsono, { id: sendSettingJson_jsono.id });
        this.setting = sendSettingJson_jsono;
        this.allsetting = false;
        process.then(function(results) {
          a1.setting = results.entity;
        });
      }
    },

    sendAllSettingJson: function() {
      var sendSettingJson_jsono = this.createSettingJson();
      sendSettingJson_jsono.allsetting = true;
      var store = new JsonRest({
        target: "rest/jfcgraphsetting/entity"
      });
      var a1 = this;
      store.query(this.getAllSettingJsonQuery()).then(function(results) {
        if (results == null || results.entity == null || results.entity.length == 0) {
          var process = store.add(sendSettingJson_jsono);
          process.then(function(results) {
            showAlertDialog(a1.getMessage("sendAllSettingSaveLabel"));
          });
        } else {
          var result = results.entity[0];
          if (results.entity.length > 1) {
              showAlertDialog(a1.getMessage("sendAllSettingOverLabel").replace("result.id", result.id));
          }
          sendSettingJson_jsono.id = result.id;
          sendSettingJson_jsono.userid = result.userid;
          var process = store.put(sendSettingJson_jsono, { id: sendSettingJson_jsono.id });
          process.then(function(results) {
            showAlertDialog(a1.getMessage("sendAllSettingSaveLabel"));
          });
        }
      });
    },

    deleteSettingJson: function() {
      if (! window.confirm(this.getMessage("deleteSettingConfirmLabel"))) {
          return;
      }
      var store = new JsonRest({
          target: "rest/jfcgraphsetting/entity"
      });
      if (this.setting == null || this.setting.id == null) {
        store.query(this.getUserSettingJsonQuery()).then(
                lang.hitch(this, "getAllSettingJson2"));
      } else {
        store.remove(this.setting.id);
        this.getAllSettingJson();
      }
    },

    getAllSettingJson2: function(results) {
      if (results == null || results.entity == null || results.entity.length == 0) {
      } else {
        var store = new JsonRest({
          target: "rest/jfcgraphsetting/entity"
        });
        var result = results.entity[0];
        store.remove(result.id);
      }
      this.getAllSettingJson();
    },

    getAllSettingJson: function() {
      var store = new JsonRest({
        target: "rest/jfcgraphsetting/entity"
      });
      var a1 = this;
      store.query(this.getAllSettingJsonQuery()).then(function(results) {
        if (results == null || results.entity == null || results.entity.length == 0) {
          a1.setDefaultValue();
        } else {
          var result = results.entity[0];
          result.id = null;
          a1.isStopChangeEvent = true;
          a1.setSettingJsonInPage(result);
          a1.isStopChangeEvent = false;
        }
        a1.makeObjects();
      });
    },

    isDifferentSetting: function(a, b) {
      var c = [ "plotTypeAxis", "plotTypeGraph", "plotTypeColor" ];
      var d = [ "screenid", "userid", "useridjshparam", "allsetting" ];
      if (this.isDifferent(a, b, c, d)) {
          return true;
      }
      for (var key in c) {
          if (this.isDifferent(a[c[key]], b[c[key]], [], [])) {
              return true;
          }
      }
      return false;
    },

    isDifferent: function(a, b, c, d) {
      for (var key in a) {
        if (array.indexOf(c, key) >= 0) {
          continue;
        }
        if (array.indexOf(d, key) >= 0) {
          continue;
        }
        if (a[key] != b[key]) {
          return true;
        }
      }
      return false;
    },

    comparatorDatasItem: function(a, b) {
      var v1 = this.getDimContent(a);
      var n1 = Number(v1);
      var v2 = this.getDimContent(b);
      var n2 = Number(v2);
      if (isNaN(n1)) {
          if (isNaN(n2)) {
              return v1.localeCompare(v2);
          } else {
              return 1;
          }
      } if (isNaN(n2)) {
          return -1;
      }
      return n1 - n2;
    },

    __getDimContent: function(dimensionItemName, item) {
      if (item[dimensionItemName].label != undefined) {
        return item[dimensionItemName].content;
      } else {
        if (item[dimensionItemName][0].id != undefined) {
          return item[dimensionItemName][0].label;
        } else {
          return item[dimensionItemName][0].content;
        }
      }
      return "";
    },

  });
});
